#include "ScannerS2HDM.h"

using namespace std;

static double Pi=acos(-1);

//WARNING: This routine assumes the two scalars that get vevs in the decompositions of the two doublets are phi[2] and phi[6]!
void PhysicalStates2HDM(MassRef & Mass, MmixingRef & Mixing,PhiRef & Phi,double & mHheavy, double & mhlight, double & mHc, double & mA, double & alpha,double & beta){
  //This routines returns the masses of the scalars in the 2HDM as well as the alpha mixing angle for the 2 CP even Higgs bosons and the beta angle between the vevs. 
  /// INPUT: 
  //       Mass - internal (vector like) array of masses
  //       Mixing - internal (matrix like) mixing matrix
  //       Phi - internal (vector like) array of field values
  /// OUTPUT: 
  //       mHheavy - Mass of the heavy CP even Higgs
  //       mhlight - Mass of the light CP even Higgs
  //       mHc - Mass of the charged Higgs
  //       mA - Mass of the CP odd Higgs
  //       alpha - mixing angle for the 2 CP even Higgs bosons
  //       beta - angle between the vevs

  mA=Mass[2]; //Mass of the pseudo scalar (numbering checked in VERBOSE mode)
  mHc=Mass[3]; //Mass of the charged Higgs (numbering checked in VERBOSE mode. Note Mass[4] is the same because it corresponds to the imaginary part).

  //Note that 
  //    --
  //    | phi[2]=Mixing[0][2].H1+Mixing[1][2].H2
  //    | phi[6]=Mixing[0][6].H1+Mixing[1][6].H2
  //    --
  double reflection=Mixing[0][2]*Mixing[1][6]-Mixing[1][2]*Mixing[0][6];

  alpha=acos(reflection*Mixing[0][2]);
  if(-Mixing[1][2]<0)//Deciding quadrant using the sign of the sine
    alpha=2*Pi-alpha;

  //Reduction to the quadrants in [-Pi/2;Pi/2] 
  if(0.5*Pi<alpha<=1.5*Pi) 
    alpha=Pi-alpha;
  else if(1.5*Pi<alpha<=2*Pi)
    alpha-=2*Pi;

  //Mass ordering
  if(Mass[0]<Mass[1]){
    alpha=0.5*Pi-abs(alpha);
    mHheavy=Mass[1];
    mhlight=Mass[0];
  }
  else{
    mHheavy=Mass[0];
    mhlight=Mass[1];
  }
  //Computing beta from VEVs
  beta=atan(Phi[6]/Phi[2]);
}
