#include"ScannerSEWPO.h"

using namespace std;

////////////////////////
///// SM functions /////
////////////////////////

double DSsm(double Mass){
  double XZ2=pow(Mass,2)/Mz2;
  /*if(XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DSsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return Psi(XZ2)/Pi;
}

double DTsm(double Mass){
  double XW2=pow(Mass,2)/Mw2,XZ2=pow(Mass,2)/Mz2;
  /*if(XW2<1|| XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DTsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
      }*/
  double term1,term2;
  if(XW2-1<1e-3)
    term1=(3-XW2)/2e0;
  else
    term1=log(XW2)/(1-XW2); 
  if(XZ2-1<1e-3)
    term2=(3-XZ2)/2e0;
  else
    term2=log(XZ2)/(1-XZ2);     

  return -3e0/16e0/Pi/sw2*XW2*(term1-term2);
}

double DUsm(double Mass){
  double XW2=pow(Mass,2)/Mw2,XZ2=pow(Mass,2)/Mz2;
  /*if(XW2<1|| XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DUsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
      }*/

  return (Psi(XW2)-Psi(XZ2))/Pi;
}

double DVsm(double Mass){
  double XZ2=pow(Mass,2)/Mz2;
  /*if(XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DVsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return (Chi(XZ2)-Psi(XZ2))/4/Pi/sw2/cw2;
}

double DWsm(double Mass){
  double XW2=pow(Mass,2)/Mw2;
  /*if(XW2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DWsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return (Chi(XW2)-Psi(XW2))/4/Pi/sw2;
}

double DXsm(double Mass){
  return 0;
}

double Psi(double A){
  return Integrator(A,PsiIntegrand)+A*((4-A)*log(A)/(1-A)-0.5)/4e0;
}

double PsiIntegrand(double x,void * params){
  double A = *((double *) params);
  return (1/2e0+(1-A/2e0)*x-x*x/2e0)*log((1-x)*(1-x)+A*x);
}

double Chi(double A){
  return Integrator(A,ChiIntegrand);
}

double ChiIntegrand(double x,void * params){
  double A = *((double *) params);
  return x*(1-x)*(1e0/((1-x)*(1-x)+A*x)-0.5*log((1-x)*(1-x)+A*x));
}

///////////////// END OF SM functions ////////////////

//////////////////////////////////
////// N-singlet functions ///////
//////////////////////////////////

double DS_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DSsm(Mass[a]);
  return result;
}

double DT_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DTsm(Mass[a]);
  return result;
}

double DU_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DUsm(Mass[a]);
  return result;
}

double DV_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DVsm(Mass[a]);
  return result;
}

double DW_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DWsm(Mass[a]);
  return result;
}

double DX_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DXsm(Mass[a]);
  return result;
}

////////// END OF n-singlets functions /////////


///////////////////////////
////// 2HDM routines //////
///////////////////////////

double DS_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DSsm(mHheavy)+F1(mA/Mz,mhlight/Mz)/2/Pi)+pow(sin(beta-alpha),2)*(DSsm(mhlight)+F1(mA/Mz,mHheavy/Mz)/2/Pi)+((2-4*cw2*sw2)*(F1(mHc/Mz,mHc/Mz)-log(mHc/Mz)/3e0)-F1(mHc/Mz,mHc/Mz))/2/Pi;
  //  return -(1/3e0*log(mHc*mHc)-pow(sin(beta-alpha),2)*Fdeltaline(mHheavy,mA)-pow(cos(beta-alpha),2)*Fdeltaline(mhlight,mA))/4/acos(-1);
}

double DT_2HDM(double mhlight, double mHheavy, double mA, double mHc,double alpha, double beta){
  return pow(cos(beta-alpha),2)*(DTsm(mHheavy)+(F0(mHc/Mz,mhlight/Mz)-F0(mA/Mz,mhlight/Mz))/16/Pi/sw2/cw2)+pow(sin(beta-alpha),2)*(DTsm(mhlight)+(F0(mHc/Mz,mHheavy/Mz)-F0(mA/Mz,mHheavy/Mz))/16/Pi/sw2/cw2)+F0(mHc/Mz,mA/Mz)/16/Pi/sw2/cw2;
  //double GF = 1.16639e-5;
  //double alphaem = 7.2973525698e-3;
  //return-(-F0(mA,mHc)+pow(sin(beta-alpha),2)*(F0(mHheavy,mA)-F0(mHheavy,mHc))+pow(cos(beta-alpha),2)*(F0(mhlight,mA)-F0(mhlight,mHc)))*sqrt(2)*GF/16e0/pow(acos(-1),2)/alphaem;
}

double DU_2HDM(double mhlight, double mHheavy, double mA, double mHc,double alpha, double beta){
  return pow(cos(beta-alpha),2)*(DUsm(mHheavy)+F1(mHc/Mw,mhlight/Mw)/2/Pi)+pow(sin(beta-alpha),2)*(DUsm(mhlight)+F1(mHc/Mw,mHheavy/Mw)/2/Pi)+(F1(mHc/Mw,mA/Mw)-2*log(mHc/Mw)/3)/2/Pi-DS_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta);
  //double GF = 1.16639e-5;
  //double alphaem = 7.2973525698e-3; 
  //return-(-F0(mA,mHc)+pow(sin(beta-alpha),2)*(F0(mHheavy,mA)-F0(mHheavy,mHc))+pow(cos(beta-alpha),2)*(F0(mhlight,mA)-F0(mhlight,mHc)))*sqrt(2)*GF/16e0/pow(acos(-1),2)/alphaem;
}

double DV_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DVsm(mHheavy)+G1(mA/Mz,mhlight/Mz)/8/Pi/sw2/cw2)+pow(sin(beta-alpha),2)*(DVsm(mhlight)+G1(mA/Mz,mHheavy/Mz)/8/Pi/sw2/cw2)+G1(mHc/Mz,mHc/Mz)*(1-4*sw2*cw2)/8/Pi/sw2/cw2;
}

double DW_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DWsm(mHheavy)+G1(mHc/Mw,mhlight/Mw)/8/Pi/sw2)+pow(sin(beta-alpha),2)*(DWsm(mhlight)+G1(mHc/Mw,mHheavy/Mw)/8/Pi/sw2)+G1(mHc/Mw,mA/Mw)/8/Pi/sw2;
}

double DX_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*DXsm(mHheavy)+pow(sin(beta-alpha),2)*DXsm(mhlight)+(cw2-sw2)*G2func(mHc*mHc/Mz2)/4/Pi;
}

double F0(double A, double B){//F0=(A^2+B^2)/2-A^2B^2Log(A^2/B^2)/(A^2-B^2)
  double SumSq=A*A+B*B,delta=(A*A-B*B)/SumSq;//Auxiliary variables which are useful to test if close to numerically difficult point when masses are degenerate
  if (abs(delta)<1e-3)
    return abs(delta)+SumSq*delta*delta*(1+delta*delta/5e0)/3e0; //Use series expansion if close to such point...
  else
    return SumSq*(0.5+0.25*(delta-1)*(delta+1)/delta*log((1+delta)/(1-delta)));//Otherwise use full expression (simplified here in terms of delta and SumSq)
}

double F1(double A, double B){
  /*if(A+B<1){
    cerr<<"ScannerS error in ScannerSEWPO.cpp: double F1(double A, double B): Loop functions for 2HDM called with masses such that corrections are not virtual (i.e. masses A+B<M_V with A,B loop particle masses and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  double * Msq;
  Msq= new double [2];
  Msq[0]=A*A;
  Msq[1]=B*B;
  double result=IntegratorDx(IntegrandF1,Msq)-1e0/6e0;
  delete Msq;
  return result;
}

double IntegrandF1(double x,void * params){
  double * Msq = (double *) params;
  double Prod=x*(1-x),ArgLog=(1-x)*Msq[0]+x*Msq[1]-Prod;
  //clog<<"msq0="<<Msq[0]<< " Msq[1]="<<Msq[1]<< endl;
  //return Prod*log(ArgLog)-(ArgLog+Prod)*log(ArgLog/(ArgLog+Prod));
  return (ArgLog+Prod)*log(ArgLog+Prod)-ArgLog*log(ArgLog);
}

double G1(double A, double B){
  /*if(A+B<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double G1(double A, double B): Loop functions for 2HDM called with masses such that corrections are not virtual (i.e. masses A+B<M_V with A,B loop particle masses and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  double * Msq;
  Msq= new double [2];
  Msq[0]=A*A;
  Msq[1]=B*B;
  double result=IntegratorDx(IntegrandG1,Msq)+1e0/6e0;
  delete Msq;
  return result;
}

double IntegrandG1(double x,void * params){
  double * Msq = (double *) params;
  double Prod=x*(1-x),ArgLog=(1-x)*Msq[0]+x*Msq[1]-Prod;
  return (ArgLog+Prod)*log(ArgLog/(ArgLog+Prod));
}

double G2func(double A){
  /*if(A<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double G2func(double A): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses A<M_V with A loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return Integrator(A,IntegrandG2func)+1e0/6e0;
}

double IntegrandG2func(double x,void * params){
  double A = *((double *) params);
  return (A*A-x*(1-x))*log(1-x*(1-x)/A/A);
}

bool CheckVirtual(double m1, double m2){
  double m1sq_Mw2=pow(m1,2)/Mw2,m1sq_Mz2=pow(m1,2)/Mz2,m2sq_Mw2=pow(m2,2)/Mw2,m2sq_Mz2=pow(m2,2)/Mz2;
  double Sw2=m1sq_Mw2+m2sq_Mw2,Sz2=m1sq_Mz2+m2sq_Mz2;
  double dw2=m1sq_Mw2-m2sq_Mw2,dz2=m1sq_Mz2-m2sq_Mz2;
  if((abs(dw2)<1 && Sw2<=(dw2*dw2+1)/2e0)||(abs(dz2)<1 && Sz2<=(dz2*dz2+1)/2e0))
    return false;
  return true;
}


