/*
This file contains pieces of code that can be used to test/use particular features in the UserAnalysis.cpp file to build your own analysis. We have divided this into sections for ease fo navigation according to what you need for your analysis:

WARNING: This file is not to be compiled as it is!!!! These are illustrative pieces of code


1) ScannerS core functions examples

2) Interface code examples
  2.1- Superiso 
  2.2- SuShi
  2.3- Hdecay
  2.4- MicrOmegas (TO DO with xSM!!!)

3) Useful model specific functions examples
  3.1- 2HDM

4) Tables and analysis utilities examples
  4.1- Accessing internal tables (TO DO)
  4.2- Generating tables to speed up scans (DOING THIS!!!!)
  4.3- Generating Standard Output files (TO DO)
  4.4- Generating Useful scripts (TO DO)

 */

/*------------------------------------------
2) Interface code examples------------------
--------------------------------------------
*/

/*-----
2.1- Superiso Interface
-------*/
//NOTE: You need to enable the superiso interface in the makefile (see manual for further instructions).

/////
// Example of how to use the interface to compute b-> s \gamma in the 2HDM and output accepted points to a text file in UserInitCalcs:

ofstream outfile;
outfile.open("test.txt");
//Loop over values on a rectangle in the (mHcharged,tanbeta) plane
for(double mHcharged=300; mHcharged<600; mHcharged+=2){
  for(double logtanbeta=0; logtanbeta<2;logtanbeta+=0.02){
    tanbeta=pow(10,logtanbeta); //generate tanbeta for a log scale
    //Create Input File for superiso
    CreateInputFileSuperiso2HDM(mHlight,mHheavy,mA,mHcharged,alpha,tanbeta,Type2HDM);
    //Example of na observable calculated with superiso
    double bsgamma=bsgamma_calculator(superisofile);
    if(bsgamma*1e4<4.1 && bsgamma*1e4 >3){//Accept observable test if in experimentally allowed range
      outfile<< tanbeta<<"\t"<<mHcharged<<endl;
    }
  }
 }
outfile.close();

///END OF EXAMPLE

/*-----
2.2- SuShi Interface
-------*/
//NOTE: You need to enable the SuShi interface in the makefile (see manual for further instructions).

//// First define the model parameters
int Type2HDM=3; //2HDM type of model (options are I,II,III,IV)
double mHlight=126; //h particle
double mHheavy=500; //H particle
double mA=300; //A particle
double mHcharged= 250; //Hc charged higgs
double alpha=0.23; //mixing angle
double beta=0.34; // beta in tanbeta

//// Process and collider info
int Hparticle=2; // Which H particle produced. Options are 0,1,2 for h,A,H
int pp_ppbar=0; //Which type of collider (0 is pp)
int order=2; // QCD order of corrections
double CM_energy=8000; // Centre of mass energy
//// Create input to interface to SusHi
CreateInputFileSusHi2HDM(Hparticle,pp_ppbar,order,CM_energy,mHlight, mHheavy,mA,mHcharged,alpha,tanbeta,Type2HDM);
//Call SusHi to compute two contributions
double xsecggh_out,errxsecggh_out,xsecbbh_out,errxsecbbh_out=0;
sushixsection_(xsecggh_out,errxsecggh_out,xsecbbh_out,errxsecbbh_out);
//Output result
std::clog <<endl <<"***** ScannerS: extracted SusHi output  ***** " << endl<< "Xsec ggH = "<<xsecggh_out <<" +/- " << errxsecggh_out << "\t Xsec bbh = " << xsecbbh_out<< " +/- " << errxsecbbh_out <<endl<< endl;
std::clog<<"mHlight="<<mHlight<<"\t mHheavy="<<mHheavy<<"\t mA="<<mA<<endl;

/// END OF EXAMPLE/////
///////////////////////


/*-----
2.2- Hdecay Interface
-------*/
//NOTE: You need to enable the Hdecay interface (see manual for further instructions).

///////
//Calling Hdecay for 2HDM to compute all total widths and branching ratios

//Define first model values to pass to Hdecay
double mA=500; //Mass of the pseudo scalar 
double mHcharged=550; //Mass of the charged Higgs 
double mHlight=125,mHheavy=400; // Masses of the light and heavy higgs 
double alpha=-0.05494;//This is the angle in the 2x2 mixing matrix
int Type2HDM=2; // Define which type of 2HD
double tanbeta=10;//Define tanbeta
double m12sq=158e2;//Define m12^2 coupling
CreateInputFileHdecay2HDM(Type2HDM,tanbeta,alpha,mHlight,mHheavy,mA,mHcharged,m12sq); // Pass information on model to Hdecay by creating an input file for Hdecay
HdecayCalc2HDM(HdecayA,HdecayHlight,HdecayHheavy,HdecayHcharged);//Fill up map variables for each particle which contain the list of branching ratios and total widths (see manual)

//Example line of how to call an element in the map (see also manual)
std::clog<<"BR(A -> b bbar) = \t"<<HdecayA["BR(A -> b bbar)"]<<"!!"<< endl;

//Header information on the model point
std::clog<<"Testing Hdecay interface for 2HDM with"<< endl;
std::clog<<"Type2HDM = "<<Type2HDM<<endl<<"tanbeta = "<<tanbeta<<endl<<"alpha = "<<alpha<<endl<<"mHlight = "<<mHlight<<endl<<"mHheavy = "<<mHheavy<<endl<<"mA = "<<mA<<endl<<"mHcharged = "<<mHcharged<<endl<<"m12sq = "<<m12sq<<endl;

//Now print out all the maps on screeb to get full information
std::clog<<"Let's print out everything in the Hdecay maps:"<< endl<<endl;
for(map<string,double>::iterator it = HdecayA.begin();it!=HdecayA.end();++it){
  std::clog<<"HdecayA[\""<<(*it).first<<"\"] = "<<(*it).second<< endl;
 }
std::clog<<endl;
for(map<string,double>::iterator it = HdecayHlight.begin();it!=HdecayHlight.end();++it){
  std::clog<<"HdecayHlight[\""<<(*it).first<<"\"] = "<<(*it).second<< endl;
 }
std::clog<<endl;
for(map<string,double>::iterator it = HdecayHheavy.begin();it!=HdecayHheavy.end();++it){
  std::clog<<"HdecayHheavy[\""<<(*it).first<<"\"] = "<<(*it).second<< endl;
 }
std::clog<<endl;
for(map<string,double>::iterator it = HdecayHcharged.begin();it!=HdecayHcharged.end();++it){
  std::clog<<"HdecayHcharged[\""<<(*it).first<<"\"] = "<<(*it).second<< endl;
 }
std::clog<<endl;

///END OF EXAMPLE


/*---------------------------------------------
3) Useful model specific functions examples----
-----------------------------------------------
*/

/*-----
3.1- Physical state ordering to 2HDM conventions
-------*/

// Routine to extract the masses and mixing angles for the 2HDM in the usual conventions (alpha in [-Pi/2,Pi/2]) and beta in [0:Pi/2] (see SusHi example for names of variables)
 PhysicalStates2HDM(Mass,Mixing,Phi,mHheavy,mHlight,mHcharged,mA,alpha,beta);
//NOTE: this routine was tested with the 2HDM implementation provided in the examples!


/*---------------------------------------------
4) Tables and analysis utilities examples------
-----------------------------------------------
*/

/////
//  4.1- Accessing internal tables 
/////

//////////////////////////////////////
//// Superiso BR(B-> X_s gamma) tables

//To use the tables in ScannerScore/data/SuperisoBsgamma2HDMType*.dat you need to add to ScannerSUser.cpp the following:

//--1 Global declaration of a TableData variable to hold the table you want to access. For example for the Type1 table add the following line after the header declarations

TableData TableSuperisoBsgammaType1;

//--2 Initialise the table in void UserInitCalcs(void) so that it is loaded before the scan starts

  // Reading superiso data table from data file
  std::clog<<"Initialising Internal table for B-> X_s gamma created with superiso..."<< endl;
  TableSuperisoBsgammaType1.create_from_file("ScannerScore/data/SuperisoBsgamma2HDMType1.dat");

//--3 Call the (interpolated) table point wherever you want with appropriate input. For example in UserAnalysis you may want to call it as follows:

  //This table depends on two variables mHcharged (the charged Higgs mass) and beta (the vev mixing angle)
  vector<double> Point(2);
  Point[0]=350; //This is mHcharged
  Point[1]=0.5; //This is beta
  std::clog<<"Calling Internal table for BR(B-> X_s gamma) created with superiso..."<< endl;
  std::clog<<"BR(B-> X_s gamma) is "<<TableSuperisoBsgammaType1(Point)<<endl<<" for mHcharged= "<<Point[0]<<" and beta = "<< Point[1] <<" (tanbeta= "<<tan(Point[1])<<")"<< endl;



// END OF EXAMPLE ////////////////////////
/////////////////////////////////////////


/////
//  4.2- Generating tables to speed up scans 
/////

//For this example you need first to define the function you need to interpolate such as:

double f1(std::vector<double> & Point,void * p){
  double out=1;
  for(int i=0;i<Point.size();++i){
    out*=exp(-Point[i]*Point[i]);
  }
  return out;
}

TableData TableTest;
// For the tables to be visible in the whole ScannerSUser.cpp functions, they must be declared outside at the begining, i.e. together with f1.

// Then... a possible code to create the table, test it, and output it is to write in UserInitCalcs:

  //////////////////////////
  // Example of how to use the tables class
  //////////////////////////
  int Nindices=3; // We will use a function of 3 variables 
  // vectors to hold ranges and number of points 
  std::vector<double> MinRanges_in(Nindices),MaxRanges_in(Nindices);
  std::vector<int> NsGrid_in(Nindices);
  //Entering values for grid...
  NsGrid_in[0]=25;
  MinRanges_in[0]=-2;
  MaxRanges_in[0]=2;
  NsGrid_in[1]=25;
  MinRanges_in[1]=-2;
  MaxRanges_in[1]=2;
  NsGrid_in[2]=25;
  MinRanges_in[2]=-2;
  MaxRanges_in[2]=2;
  //Creating the table for later use
TableTest.create_from_scratch(MinRanges_in,MaxRanges_in,NsGrid_in,f1,NULL);
  vector<double> Point1(Nindices);
  Point1[0]=0.2536;
  Point1[1]=-0.5698;
  Point1[2]=-0.2;
  //Calling the table directly to get interpolated value
std::clog<<"Testing the point ("<<Point1[0]<<","<<Point1[1]<<","<<Point1[2]<<"). \n Interpolated value="<<TableTest(Point1)<<"\n exact value= "<< f1(Point1,NULL)<<" \n err="<< abs(TableTest(Point1)-f1(Point1,NULL))<<endl;
  //Testing the quality of the interpolation by looping over a random sample of pointsin the interpolation domain
  RandGen Rgen1;
  double err=0,errlocal;
  std::clog<<"Testing 10000 points..."<< endl;
  for(int i=0;i<10000;++i){
    Point1[0]=Rgen1(-2,2);
    Point1[1]=Rgen1(-2,2);
    Point1[2]=Rgen1(-2,2);
    errlocal=abs(TableTest(Point1)-f1(Point1,NULL));
    if(errlocal>err)
      err=errlocal;
  }
  std::clog<<"MAX ERR found = "<<err<<endl;

  std::clog<<"Testing output interface by: 1) saving twice to the same output file test.txt (to check warnings. Run both cases Y and N after you get the warning), 2) reading from test.txt and writing to test2.txt"<< endl;

  TableTest.save_table("test.txt");
  TableTest.save_table("test.txt");

  ifstream input("test.txt");
  input >>TableTest;
  TableTest.save_table("test2.txt");
  
//// END OF EXAMPLE





